#include <time.h>
#include <stdlib.h>
#include "ads.h"
#include "conf.h"
#include "main.h"
#include "heap.h"
#include "log.h"

static void free_ad( struct ad_t * ad) {
    size_t i;

    if ( ad != NULL) {
	safefree(ad->match_str);
	for (i = 0; i < ad->n_refs; i++)
	    safefree(ad->refs[i]);
    }
}

static struct ad_t * ad_create_rec(const char * type, const char * t, const char * refs) {
    struct ad_t * ad = NULL;

    if ((ad = safemalloc(sizeof(struct ad_t))) != NULL) {
	char * ptr = refs, * ptr1;

	memset(ad, 0, sizeof(struct ad_t));
	if (strcmp(type, "yandex") == 0)
	    ad->type = AD_YANDEX;
	else if (strcmp(type, "google") == 0)
	    ad->type = AD_GOOGLE;
	else {
	    safefree(ad);
	    ad = NULL;
	    goto ACR_done;
	}
	if ( t[0] == 'h')
	    ad->match = AD_MATCH_HOST;
	else if ( t[0] == 'u')
	    ad->match = AD_MATCH_URL;
	else {
	    safefree(ad);
	    ad = NULL;
	    goto ACR_done;
	}
	if ((ad->match_str = safemalloc(strlen(t))) != NULL) {
		strcpy(ad->match_str, &t[2]);
	} else {
	    safefree(ad);
	    ad = NULL;
	    goto ACR_done;
	}
	while (ad->n_refs < AD_MAX_REFS) {
	    if ((ptr1 = strstr(ptr, ",")) != NULL) {
		if ((ad->refs[ad->n_refs] = safemalloc(ptr1-ptr+1)) != NULL) {
		    char buf[256];

		    memset(ad->refs[ad->n_refs], 0, ptr1-ptr+1);
		    memset(buf, 0, sizeof(buf));
		    memcpy(buf, ptr, ptr1-ptr);
		    strstrip(ad->refs[ad->n_refs], ptr1-ptr+1, buf);
		    ptr = ptr1 + 1;
		    ad->n_refs ++;
		} else {
		    free_ad( ad);
		    ad = NULL;
		    goto ACR_done;
		}
	    } else {
		if ((ad->refs[ad->n_refs] = safemalloc(strlen(ptr)+1)) != NULL) {
		    char buf[256];

		    memset(ad->refs[ad->n_refs], 0, strlen(ptr)+1);
		    memset(buf, 0, sizeof(buf));
		    strcpy(buf, ptr);
		    strstrip(ad->refs[ad->n_refs], strlen(ptr)+1, buf);
		    ad->n_refs ++;
		    break;
		} else {
		    free_ad( ad);
		    ad = NULL;
		    goto ACR_done;
		}
	    }
	}
    }
ACR_done:
    return ad;
}

void ad_add_rec(const char * type, const char * t, const char * refs) {
    struct ad_t * ad = ad_create_rec( type, t, refs);

    if ( ad != NULL) {
	ad->next = config.ads_list;
	config.ads_list = ad;
    }
}

static char * get_random_ad( struct ad_t * ad) {
    char * ptr = NULL;

    if ((ad != NULL) && (ad->n_refs > 0)) {
	if ( ad->n_refs == 1)
	    return ad->refs[0];
	else {
	    srand(time(NULL));
	    return ad->refs[rand() % ad->n_refs];
	}
    }
    return ptr;
}

void free_ads( struct ad_t * ad) {
    while (ad) {
	struct ad_t *tmp = ad;
	ad = ad->next;
	free_ad(tmp);
    }
}

static struct ad_t * get_ads_by_url( const char * url, struct ad_t * ad) {
    struct ad_t * tmp = ad;

    while ((tmp != NULL) && (url != NULL)) {
	if ((tmp->match == AD_MATCH_URL) && (strstr(url, tmp->match_str) != NULL))
	    return tmp;
	tmp = tmp->next;
    }
    return NULL;
}

static struct ad_t * get_ads_by_host( const char * host, struct ad_t * ad) {
    struct ad_t * tmp = ad;

    while ((tmp != NULL) && (host != NULL)) {
	if ((tmp->match == AD_MATCH_HOST) && (strcmp(host, tmp->match_str) == 0))
	    return tmp;
	tmp = tmp->next;
    }
    return NULL;
}

static int recombine_referer_header(hashmap_t hashofheaders, const char * referer) {
        ssize_t len;
        char *data;
        int ret = -1;

        len = hashmap_entry_by_key (hashofheaders, "referer", (void **) &data);
        if (len > 0) {
	    if ((ret = hashmap_insert(hashofheaders, "X-STSN-ref", data, len)) >= 0)
		ret = hashmap_remove(hashofheaders, "referer");
	}
	if ( ret >= 0) {
	    ret = hashmap_insert(hashofheaders, "Referer", referer, strlen(referer));
	}
	return ret;
}

static int recombine_origin_header(hashmap_t hashofheaders, const char * referer) {
        ssize_t len;
        char hostname[512];
        char *data;
        int ret = -1;

        len = hashmap_entry_by_key (hashofheaders, "origin", (void **) &data);
        if (len > 0) {
	    if ((ret = hashmap_insert(hashofheaders, "X-STSN-or", data, len)) >= 0)
		ret = hashmap_remove(hashofheaders, "origin");
	}
	if ( ret >= 0) {
	    snprintf(hostname, sizeof(hostname), "%s", referer);
	    len = strlen(hostname);
	    if ((len > 0) && (hostname[len-1] == '/'))
		hostname[--len] = 0;
	    ret = hashmap_insert(hashofheaders, "Origin", hostname, len);
	}
	return ret;
}

static char to_hex( char code) {
    static char hex[] = "0123456789ABCDEF";
    return hex[code & 0xf];
}

static char * url_encode( char * dest, size_t size, const char * src) {
    char * ptr = src;
    size_t i = 0;

    if ((dest == NULL) || (src == NULL))
	return;
    while ((i < size) && *ptr) {
	if (isalnum(*ptr) || (*ptr == '-') || (*ptr == '_') || (*ptr == '.') || (*ptr == '~'))
	    dest[i++] = *ptr;
	else if (*ptr == ' ')
	    dest[i++] = '+';
	else {
	    dest[i++] = '%';
	    dest[i++] = to_hex(*ptr >> 4);
	    dest[i++] = to_hex(*ptr & 0xf);
	}
	ptr++;
    }
    dest[i] = 0;
    return dest;
}

static char * recombine_anyan_url(const char * url, const char * referer, const char * site) {
        ssize_t len, len1, slen;
        char hostname[512];
        char *data, *new_url = NULL;
	int use_site = 0, use_b64 = 0;

	if ((data = strstr(url, "target-ref=http://")) != NULL) {
	    data += 18;
	} else {
	    if ((data = strstr(url, "target-ref=http%3A%2F%2F")) != NULL) {
		data += 24;
		use_b64 = 1;
	    } else {
		if ((data = strstr(url, "page-url=http://")) != NULL) {
		    data += 16;
		    use_site = 1;
		} else {
		    if ((data = strstr(url, "page-url=http%3A%2F%2F")) != NULL) {
			data += 22;
			use_site = 1;
			use_b64 = 1;
		    }
		}
	    }
	}
	if ( data != NULL) {
	    if ( use_b64)
		url_encode(hostname, sizeof(hostname), ((use_site==1)?site:referer)+7);
	    else
		snprintf(hostname, sizeof(hostname), "%s", ((use_site==1)?site:referer)+7);
	    slen = strlen(hostname);
	    if ((slen > 0) && (hostname[slen-1] == '/'))
		hostname[--slen] = 0;
	    len = strlen(url);
	    len1 = data - url;
	    if ((new_url = safemalloc(len+slen+1)) == NULL)
                goto YANR_done;
	    memcpy(new_url, url, len1);
	    memcpy(new_url+len1, hostname, slen);
	    if ((data = strstr(data, "&")) != NULL)
		strcpy(new_url+len1+slen, data);
	    else
		new_url[len1+slen] = 0;
	}
YANR_done:
	return new_url;
}

static char * recombine_gad_url(const char * url, const char * referer) {
        ssize_t len, len1, slen;
        char hostname[512];
        char *data, *new_url = NULL;

	if ((data = strstr(url, "url=http://")) != NULL) {
	    data += 11;
	} else {
	    if ((data = strstr(url, "url=http%3A%2F%2F")) != NULL) {
		data += 17;
	    }
	}
	if ( data != NULL) {
	    url_encode(hostname, sizeof(hostname), referer+7);
	    slen = strlen(hostname);
	    if ((slen > 0) && (hostname[slen-1] == '/'))
		hostname[--slen] = 0;
	    len = strlen(url);
	    len1 = data - url;
	    if ((new_url = safemalloc(len+slen+1)) == NULL)
                goto GADR_done;
	    memcpy(new_url, url, len1);
	    memcpy(new_url+len1, hostname, slen);
	    if ((data = strstr(data, "&")) != NULL)
		strcpy(new_url+len1+slen, data);
	    else
		new_url[len1+slen] = 0;
	}
GADR_done:
	return new_url;
}


static void prepare_ref( char * buf, size_t size, const char * url) {
    char * ptr = strstr(url, ".");

    memset( buf, 0, size);
    if ( ptr != NULL) {
	if ((ptr = strstr(ptr, "/")) != NULL) {
	    memcpy(buf, url, ptr-url+1);
	}
    }
}

char * ad_process_url( const char * host, const char * url, hashmap_t hashofheaders) {
	char * new_url = NULL;
	struct ad_t * ad = get_ads_by_host(host, config.ads_list);

	if (ad == NULL)
	    ad = get_ads_by_url(url, config.ads_list);

	if ( ad != NULL) {
	    char * referer = get_random_ad( ad);

	    if ( referer != NULL) {
		char ref_url[512];

		prepare_ref(ref_url, sizeof(ref_url), referer);
		recombine_referer_header(hashofheaders, ref_url);
		recombine_origin_header(hashofheaders, ref_url);
		if ( ad->type == AD_YANDEX) {
		    new_url = recombine_anyan_url(url, referer, ref_url);
		} else if (ad->type == AD_GOOGLE) {
		    new_url = recombine_gad_url(url, referer);
		}
	    }
	}
	return new_url;
}
